@@ -1,6 +1,7 @@
 from bank_sync.Resources.resource import Resource
 from bank_sync.APIs.utils.generate_code import get_code
 from datetime import datetime
+from datetime import date
 
 
 class Accounts(Resource):
@@ -8,39 +9,28 @@ class Accounts(Resource):
     urls = {}
 
     def set_bank_id(self, bank_id):
-        super().set_bank_id(bank_id)
-        return self._set_urls()
+        super().set_action(action = 'accounts')
+        return super().set_bank_id(bank_id)
 
-    def _set_urls(self):
+    def balance(self, payload=None):
 
-        if super().get_bank_id() in super().THIRD_PARTY_BANKING.keys():
-            self.urls["read"] = f"/v1/{super().THIRD_PARTY_BANKING.get(super().get_bank_id(),0)}"
-        else:
-            self.urls["read"] = f"/accounts/{super().get_bank_id()}"
+        return super().read(payload=payload)
 
-        super().set_urls(self.urls)
+    def mini_statement(self, payload=None):
 
-        return self
+        return super().read(payload=payload)
 
-    def balance(self, payload=None, endpoint='/get_balance'):
+    def full_statement(self, payload=None):
 
-        return super().read(payload=payload, endpoint=f'{self.urls["read"]}{endpoint}')
+        return super().read(payload=payload)
 
-    def mini_statement(self, payload=None, endpoint='/get_mini_statement'):
+    def account_validation(self, payload=None):
 
-        return super().read(payload=payload, endpoint=f'{self.urls["read"]}{endpoint}')
+        return super().read(payload=payload)
 
-    def full_statement(self, payload=None, endpoint='/get_full_statement'):
+    def account_transactions(self, payload=None):
 
-        return super().read(payload=payload, endpoint=f'{self.urls["read"]}{endpoint}')
-
-    def account_validation(self, payload=None, endpoint='/get_validation'):
-
-        return super().read(payload=payload, endpoint=f'{self.urls["read"]}{endpoint}')
-
-    def account_transactions(self, payload=None, endpoint='/get_transactions'):
-
-        return super().read(payload=payload, endpoint=f'{self.urls["read"]}{endpoint}')
+        return super().read(payload=payload)
 
     def serialize(self, payload=None, operation=None):
 
@@ -56,28 +46,35 @@ class Accounts(Resource):
             # If bank_id is COOP
             if super().get_bank_id() == super().COOP:
                 data.update({
-                    "MessageReference": f'{get_code(length=14)}'
-                })
-                data.update({
+                    "MessageReference": f'{get_code(length=14)}',
                     "AccountNumber": f'{payload.get("account_number", "")}'
                 })
             # If bank_id is EQUITY
             elif super().get_bank_id() == super().EQUITY:
                 data.update({
-                    "country_code": payload.get("country_code", "")
-                })
-                data.update({
+                    "country_code": payload.get("country_code", ""),
                     "account_number": payload.get("account_number", "")
                 })
             # If bank_id is NCBA
             elif super().get_bank_id() == super().NCBA:
                 pass
+            # If bank_id is UBA
+            elif super().get_bank_id() == super().UBA:
+                data.update({
+                    "PROCESSING_CODE": 31,
+                    "COUNTRY_CODE": payload.get("country_code", ""),
+                    "DR_ACCT_NUM": payload.get("account_number", "")
+                })
+            # If bank_id is DTB
+            elif super().get_bank_id() == super().DTB:
+                if operation == super().ACCOUNT_VALIDATION:
+                    data.update({
+                        "account_number": payload.get("account_number", "")
+                    })
             # If bank_id is THIRD_PARTY_BANKING
             elif super().get_bank_id() in super().THIRD_PARTY_BANKING.keys():
                 data.update({
-                    "businessCode": payload.get("client_id", "")
-                })
-                data.update({
+                    "businessCode": payload.get("business_code", ""),
                     "accountIds": [payload.get("account_number", "")]
                 })
 
@@ -116,17 +113,26 @@ class Accounts(Resource):
             # If bank_id is NCBA
             elif super().get_bank_id() == super().NCBA:
                 pass
+            # If bank_id is UBA
+            elif super().get_bank_id() == super().UBA:
+                data.update({
+                    "PROCESSING_CODE": 93,
+                    "COUNTRY_CODE": payload.get("country_code", ""),
+                    "DR_ACCT_NUM": payload.get("account_number", ""),
+                    "TRAN_CRNCY_CODE": payload.get("currency_code", ""),
+                    "STMT_QUERY_PARAMS": {
+                        "START_DATE": payload.get("start_date", date.today().strftime('%Y%m%d')).replace("-", ""),
+                        "END_DATE": payload.get("end_date", date.today().strftime('%Y%m%d')).replace("-", ""),
+                        "NUM_TRANS": payload.get("limit", 5)
+                    }
+                })
 
         elif operation == super().ACCOUNT_TRANSACTIONS:
             # If bank_id is COOP
             if super().get_bank_id() == super().COOP:
                 data.update({
-                    "MessageReference": f'{get_code(length=14)}'
-                })
-                data.update({
-                    "AccountNumber": f'{payload.get("account_number", "")}'
-                })
-                data.update({
+                    "MessageReference": f'{get_code(length=14)}',
+                    "AccountNumber": f'{payload.get("account_number", "")}',
                     "NoOfTransactions": f'{payload.get("limit", 1)}'
                 })
             # If bank_id is EQUITY
@@ -139,19 +145,17 @@ class Accounts(Resource):
             # If bank_id is THIRD_PARTY_BANKING
             elif super().get_bank_id() in super().THIRD_PARTY_BANKING.keys():
                 data.update({
-                    "businessCode": payload.get("client_id", "")
-                })
-                data.update({
-                    "accountIds": [payload.get("account_number", "")]
-                })
-                data.update({
-                    "limit": payload.get("limit", 1)
-                })
-                data.update({
-                    "startDate": payload.get("start_date", "")
-                })
-                data.update({
-                    "endDate": payload.get("end_date", "")
+                    "businessCode": payload.get("business_code", ""),
+                    "narration": payload.get("narration", ""),
+                    "type": payload.get("type", ""),
+                    "paginate": payload.get("paginate", True),
+                    "startDate": payload.get("start_date", ""),
+                    "endDate": payload.get("end_date", ""),
+                    "limit": payload.get("limit", 1),
+                    "total": payload.get("total", 0),
+                    "offset": payload.get("offset", 0),
+                    "accountIds": [payload.get("account_number","")],
+
                 })
 
         data.update(payload.get("additional_properties", {}))
@@ -176,9 +180,23 @@ class Accounts(Resource):
 
                 data["message"] = response_data.get("message", "")
                 data["code"] = response_data.get("code", -1111111)
-                data["balance"] = response_data.get("data", {}).get("balances",[])[0].get("amount",-1)
+                if bool(response_data.get("data", {}).get("balances",[])):
+                    data["balance"] = response_data.get("data", {}).get("balances",[])[0].get("amount",-1)
+                else:
+                    data["balance"] = 0
+            # If bank_id is UBA
+            elif super().get_bank_id() == super().UBA:
+
+                data["message"] = ''
+                data["code"] = int(float(response_data.get("sendTransactionResponse", {}).get("return", {}).get("C24TRANRES", {}).get("ACTION_CODE", "-1111111")))
+                data["balance"] = float(response_data.get("sendTransactionResponse", {}).get("return", {}).get("C24TRANRES", {}).get("AVAILABLE_BALANCE", -1))
+                if data["code"]:
+                    data["message"] = "Transaction Failed"
+                else:
+                    data["message"] = "Success"
 
         elif super().get_operation() == super().MINI_STATEMENT or super().get_operation() == super().ACCOUNT_TRANSACTIONS:
+            
              
             if super().get_bank_id() == super().COOP:
 
@@ -190,8 +208,10 @@ class Accounts(Resource):
 
                 for i in range(len(transactions)):
                     transaction = {
+                        "currency_code": "KES",
                         "date": transactions[i].get("TransactionDate", ""),
                         "description": transactions[i].get("Narration", ""),
+                        "reference": transactions[i].get("TransactionReference", "")
                     }
 
                     if transactions[i].get("TransactionType", "") == "D":
@@ -214,12 +234,12 @@ class Accounts(Resource):
                     data["code"] = f'{response_data.get("code", -111111)}'
                     data["transactions"] = []
 
-                    transactions = response_data.get(
-                        "data", {}).get("transactions", [])
+                    transactions = response_data.get("data", {}).get("transactions", [])
 
                     for i in range(len(transactions)):
 
                         transaction = {
+                            "currency_code": response_data.get("data", {}).get("currency", "KES"),
                             "date": datetime.fromisoformat(
                                 transactions[i].get("date", "") + '+00:00'
                             ).strftime('%Y-%m-%d %H:%M:%S'),
@@ -246,6 +266,17 @@ class Accounts(Resource):
                 customer = response_data.get("data", {}).get("customer", [])
                 if len(customer) > 0:
                     data["account_name"] = customer[0].get("name", "")
+            
+            # If bank_id is DTB
+            elif super().get_bank_id() == super().DTB:
+                if isinstance(response_data,list):
+                    data["message"] = response_data[0]
+                    data["code"] = -1111111
+                    data["account_name"] = ""
+                else:
+                    data["message"] = response_data.get("content", {}).get("account_description","")
+                    data["code"] = 0
+                    data["account_name"] = response_data.get("content", {}).get("customer_name","")
 
         elif super().get_operation() == super().FULL_STATEMENT:
 
@@ -258,7 +289,8 @@ class Accounts(Resource):
                 transactions = response_data.get("Transactions", [])
 
                 for i in range(len(transactions)):
-                    transaction = {
+                    transaction = {                        
+                        "currency_code": "KES",
                         "date": transactions[i].get("TransactionDate", ""),
                         "description": transactions[i].get("Narration", ""),
                     }
@@ -290,6 +322,7 @@ class Accounts(Resource):
                 for i in range(len(transactions)):
 
                     transaction = {
+                        "currency_code": response_data.get("data", {}).get("currency", "KES"),
                         "date": datetime.fromisoformat(
                             transactions[i].get("date", "") + '+00:00'
                         ).strftime('%Y-%m-%d %H:%M:%S'),
@@ -301,6 +334,34 @@ class Accounts(Resource):
                     }
 
                     data["transactions"].append(transaction)
+            # If bank_id is UBA
+            elif super().get_bank_id() == super().UBA:
+
+                data["message"] = ''
+                data["code"] = int(float(response_data.get("sendTransactionResponse", {}).get("return", {}).get("C24TRANRES", {}).get("ACTION_CODE", "-1111111")))
+                if data["code"]:
+                    data["message"] = "Transaction Failed"
+                else:
+                    data["message"] = "Success"
+                data["transactions"] = []
+
+                transactions = response_data.get("sendTransactionResponse", {}).get("return", {}).get("C24TRANRES", {}).get("TRANS_INFO", {}).get("TRAN", {}).get("element", [])
+
+                for i in range(len(transactions)):
+                    transaction = {
+                        "date": datetime.strptime(f'{transactions[i].get("DATE_POSTED", None)}', '%Y%d%m%H%M%S').strftime('%Y-%m-%d %H:%M:%S'),
+                        "description": transactions[i].get("NARRATION", ""),
+                        "amount": transactions[i].get("TRAN_AMT", -1),
+                        "reference": transactions[i].get("TRAN_ID", ""),
+                        "running_balance": transactions[i].get("BALANCE", -1),
+                    }
+
+                    if transactions[i].get("PART_TRAN_TYPE", "") == "D":
+                        transaction["type"] = "Debit"
+                    elif transactions[i].get("PART_TRAN_TYPE", "") == "C":
+                        transaction["type"] = "Credit"
+
+                    data["transactions"].append(transaction)
 
 
         if bool(data):
